/******************************************************************************
* Include files
******************************************************************************/
#include <avr/io.h>
#include "usart.h"
#include "led.h"
#include "lin.h"

/******************************************************************************
* Macros
******************************************************************************/
#define F_CPU  8000000UL  // 8 MHz

#include <avr/delay.h>

#define BAUD 19200
#define MYUBRR F_CPU/16/BAUD-1



#define STATE_WAITING 0
#define STATE_HEADER 1
#define STATE_DATA 2
#define STATE_CHK 3
#define SIGNAL_NUM 10
/******************************************************************************
* Constants
******************************************************************************/


/******************************************************************************
* Global Variables
******************************************************************************/
volatile uint8_t id=0x30;

static uint8_t  EBS_0x30[EBS_30_DLC];
static uint8_t  EBS_0x31[EBS_31_DLC];


static uint16_t signals[SIGNAL_NUM];

static uint8_t lin_state=STATE_WAITING;
/******************************************************************************
* External Variables
******************************************************************************/


/******************************************************************************
* Local Function Declarations
******************************************************************************/
uint8_t lin_checksum(uint8_t data_byte_num, uint8_t lin_bytes[]);

/******************************************************************************
* Local Function Definitions
******************************************************************************/


/******************************************************************************
* Global Function Definitions
******************************************************************************/
/******************************************************************************
* Function:      void LIN_Init (void)
* Description:   Initialize LIN 
* Input:         
* Output:
* Notes:
******************************************************************************/
void LIN_Init(void)
{
	USART0_Init(MYUBRR);
}

/******************************************************************************
* Function:      void LIN_process_bytes_task(void)
* Description:   process bytes
* Input:        
* Output:
* Notes:
******************************************************************************/
void LIN_process_bytes_task(void)
{
	uint8_t rx_data=USART0_RecieveByte();
	static uint8_t cnt=0;
	static uint8_t checksum, rxchecksum;
	static uint8_t header=255;
	static uint8_t id_data[9];
	static uint8_t dlc;
	
	switch (lin_state)
	{
		case STATE_WAITING:
		if (rx_data==0x55)
		lin_state=STATE_HEADER;
		break;
		case STATE_HEADER:
		if((rx_data==EBS_30_PID) ||(rx_data==EBS_31_PID))
		{
			id_data[cnt++]=(uint8_t)rx_data;
			lin_state=STATE_DATA;
			
		}
		else
		lin_state=STATE_WAITING;
		break;
		case STATE_DATA:
		switch (id_data[0])
		{
			case EBS_30_PID:
			if (cnt<EBS_30_DLC+1)
			id_data[cnt++]=(uint8_t)rx_data;
			else
			{
				cnt=0;
				lin_state=STATE_CHK;
				dlc=EBS_30_DLC;
				rxchecksum=rx_data;
			}
			break;
			case EBS_31_PID:
			if (cnt<EBS_31_DLC+1)
			id_data[cnt++]=(uint8_t)rx_data;
			else
			{
				cnt=0;
				lin_state=STATE_CHK;
				dlc=EBS_31_DLC;
				rxchecksum=rx_data;
			}
			break;
			default:
			cnt=0;
			header=255;
			lin_state=STATE_WAITING;
			break;
		}
		break;
		case STATE_CHK:
		//rxchecksum=(uint8_t)rx_data;
		checksum=lin_checksum(dlc,id_data);
		if (rxchecksum==checksum)
		{
		
			switch (id_data[0])
			{
				case EBS_30_PID:
				memcpy(EBS_0x30,&id_data[1],dlc);
				LED_TOOGLE(LED6);
				break;
				case EBS_31_PID:
				memcpy(EBS_0x31,&id_data[1],dlc);
				LED_TOOGLE(LED7);
				break;
				default:
				break;
			}
		}
		else
		{
			LED_TOOGLE(LED4);
		}
		header=255;
		lin_state=STATE_WAITING;
		break;
	}	
}

/******************************************************************************
* Function:      void LIN_send_headers_task(void)
* Description:   send headers
* Input:
* Output:
* Notes:
******************************************************************************/
void LIN_send_headers_task(void)
{
	switch(id)
	{
		case 0x30:
		{
			lin_header(0xB2);//30
			LED_TOOGLE(LED0);
			id=0x31;
			break;
		}
		case 0x31:
		{
			lin_header(0xF3);//31
			LED_TOOGLE(LED1);
			id=0x30;
			break;
		}
	}
}

/******************************************************************************
* Function:      void LIN_task(void)
* Description:
* Input:
* Output:
* Notes:
******************************************************************************/
/******************************************************************************
* Function:         uint16_t get_signal(uint8_t signal_num)
* Description:
* Input:
* Output:
* Notes:
******************************************************************************/
uint16_t get_signal(uint8_t signal_num)
{
	//voltage
	signals[0]=(EBS_0x30[0] | ((uint16_t)(EBS_0x30[1] & 0b01111111))<<7);



	if (signal_num>SIGNAL_NUM)
	{
		return 0xFFFF;
	}
	else
	{
		return signals[signal_num]; 
	}
	  
}

void lin_header(uint8_t ID)
{
	
	USART0_Init(80);
	
	USART0_TransmitByte('\0');
	
	_delay_us(700);
	USART0_Init(MYUBRR);//2
	
	USART0_TransmitByte(0x55);//Sync field
	USART0_TransmitByte(ID);//ID
}

/******************************************************************************
* Function:         uint8_t lin_checksum(uint8_t data_byte_num, uint8_t lin_bytes[])
* Description:
* Input:
* Output:
* Notes:
******************************************************************************/
uint8_t lin_checksum(uint8_t data_byte_num, uint8_t lin_bytes[])
{
	uint16_t checksum=0;
	uint8_t i=0;
	
	while(i<=data_byte_num)
	{
		checksum+=lin_bytes[i++];
		if (checksum>=256)
			checksum-=255;
	}
	
	return ~((uint8_t)checksum);
}

/******************************************************************************
* Interrupt Routines
******************************************************************************/





